/**
 * Copyright (c) 2014 panStamp <contact@panstamp.com>
 * 
 * This file is part of the panStamp project.
 * 
 * panStamp  is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 * 
 * panStamp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with panStamp; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 
 * USA
 * 
 * Author: Daniel Berenguer
 * Creation date: 07/23/2014
 */

/**
 * Basic radio example
 *
 * Each node acts simultaneously as sender and receiver. Each node sends a packet
 * every five senconds and toggles a LED each time a packet is received
 */
#include <HardwareSerial.h>
#include <cc430radio.h>
#define RFCHANNEL        9       // Let's use channel 0
#define SYNCWORD1        0xD3    // Synchronization word, high byte
#define SYNCWORD0        0x91    // Synchronization word, low byte
#define SOURCE_ADDR      4       // Sender address
#define DESTINATION_ADDR 5       // Receiver address

CCPACKET txPacket;  // packet object
CCPACKET rxPacket;
byte count = 0;
int voie = 20;

/**
 * This function is called whenever a wireless packet is received
 */
void rfPacketReceived(CCPACKET *packet)
{   
    // The LED will toggle when a new packet is received
    digitalWrite(LED, !digitalRead(LED));

    if (packet->length > 1)
    {   
        if (packet->data[2] == 0x33)
        {
            digitalWrite(voie, HIGH);
            Serial.println("Chaudiere en marche");
        }
        // Si recoit commande 2 de l'interface
        else if (packet->data[2] == 0x34)
        {
            digitalWrite(voie, LOW);
            Serial.println("Chaudiere arretee");
        }
    }
}

void setup()
{
    Serial.begin(38400);
    // Setup LED output pin
    pinMode(LED, OUTPUT);
    pinMode(voie, OUTPUT);
  
    digitalWrite(LED, LOW);
    Serial.print("Setup carte chaudiere");

    panstamp.radio.setChannel(RFCHANNEL);
    panstamp.radio.setSyncWord(SYNCWORD1, SYNCWORD0);
    panstamp.radio.setDevAddress(SOURCE_ADDR);
    panstamp.radio.setCCregs();

    // Let's disable address check for the current project so that our device
    // will receive packets even not addressed to it.
    panstamp.radio.disableAddressCheck();

    // Declare RF callback function
    panstamp.setPacketRxCallback(rfPacketReceived);

    // Redémarrage du module radio sinon pas de réception RF
    MRFI_STROBE_IDLE_AND_WAIT(); 
    Strobe(RF_SFRX);
    Strobe(RF_SRX); 
}

void loop()
{
    delay(500);
}

