#include <Arduino.h>
#include "dompi.h" 
void setConfigRF(unsigned char* conf);
void rfPacketReceived(CCPACKET *packet);
void setup();
void loop();
void lectureCommandeSerie();
void traiteCommande();
#line 1 "src/dompi.ino"
//#include "dompi.h" 

/* Constantes */
// le canal RF
#define CANAL			9

// les mots de synchro
#define SYNCWORD1		0xD3
#define SYNCWORD2		0x91

// L'adresse du panstamp côté compteur
#define SOURCE			1

// L'adresse du pantsamp côté box
#define DESTINATION		2

#define N 			10
#define LENGTH_DATA		8
#define LENGTH_MEMORY_ARRAY	4

// La conf RF
unsigned char conf[] = {CANAL, SYNCWORD1, SYNCWORD2, SOURCE};

// La liste des puissances d'émission dans l'ordre croissant
unsigned char puissances[] = {0x51, 0x8E, 0x60, 0x50, 0x8D, 0x8C, 0x8B, 0x8A, 0xCF, 0x89, 0x88, 0x87, 0x86, 0x85};

// le paquet reçu
EMISSION rxEmission = EMISSION(DESTINATION, SOURCE);

// le paquet émis
EMISSION txEmission = EMISSION(SOURCE, DESTINATION);

int reveil = 0;
int temperature;
char *temperature2;
float temperature3 = 0;
int longueur = 0;

int passage = 0;

typedef struct memoire
{
	int longueur;
	unsigned char donnees[LENGTH_DATA];
};

memoire tableaumemoire[LENGTH_MEMORY_ARRAY];


// Modification du paramétrage RF
void setConfigRF(unsigned char* conf)
{
	// chanel compris entre 0 et 9
	if (conf[0] >= 0 && conf[0] <= 9)
	{
		panstamp.radio.setChannel(conf[0]);
	}
	panstamp.radio.setSyncWord(conf[1], conf[2]);
	// 0 correspond à l'adresse de broadcast. Elle n'est donc pas attribuee
	if (conf[3] > 0)
	{
		panstamp.radio.setDevAddress(conf[3]);
	}
	panstamp.radio.setCCregs();
}


// Le traitement pour chaque paquet RF reçu
void rfPacketReceived(CCPACKET *packet)
{
	rxEmission.ajoute(*packet);
}

// Initialisation du panstamp
void setup()
{		
	// Initialisation de la conf RF
	setConfigRF(conf);

	// Initialisation du port série
	Serial.begin(57600);
	// La fonction à appeler lors de la réception d'un paquet RF
	panstamp.attachInterrupt(rfPacketReceived);
        panstamp.rxOn(); 
}

// La boucle principale
void loop()
{
	// Traitement du paquet reçu (lorsqu'il est complet)
	if (rxEmission.complet && !rxEmission.traite)
	{
		// Passage du numéro de commande
		// Serial.write(rxEmission.commande);
		// Passage du début de texte
		// Passage des paramètres
		//Serial.write(rxEmission.donnees, rxEmission.longueur);

                // Il reçoit la commande de réveil
		if (rxEmission.commande == 0x31)
                {
			reveil = 1;
		}
		// Il stocke la température envoyée par panstamp (2)
 		else if (rxEmission.commande == 0x32)
		{
			longueur = strlen((char *)rxEmission.donnees);
		        Serial.write(longueur+1);
		        Serial.write(rxEmission.commande);
		        Serial.write(rxEmission.donnees,longueur);
		} 

		// On est prêt à recevoir un nouveau paquet
		rxEmission.traite = true;
	}
	// Lecture d'une commande sur le port série (plus tard interface graphique)
	lectureCommandeSerie();
	// Si une commande a été lue (et est complète) sur le port série, on la traite
        // Envoi seulement si panstamp (2) est reveillé
		
        // S'il n'y a pas eu de commande et qu'il est reveillé.
	if(reveil) traiteCommande();
}

// Lecture d'une commande transmise sur le port série
void lectureCommandeSerie()
{
	// De quoi contenir un caractère lu
	unsigned char car;

	// Si il y a un caractère à lire
 	if (Serial.available() > 0)
  	{
    		// Disable wireless reception
    		//panstamp.rxOff();
    		car = Serial.read();

    		//Serial.write(car);

		// Gestion d'un débordement
		// TODO : Que fait-on? Pour le moment on supprime la dernière commandes reçues
    	        if (tableaumemoire[passage].longueur >= LENGTH_DATA-1)
    		{
      			tableaumemoire[passage].longueur = 0;
    		}
	    	else if (car == '\n')  /* Caractère retour chariot */
	    	{
		      	//panstamp.rxOn();
			if (tableaumemoire[passage].longueur > 0)
			{
		      		passage++;
			}
	    	}
	    	else
	    	{
	      		tableaumemoire[passage].donnees[tableaumemoire[passage].longueur] = car; 
	      		tableaumemoire[passage].longueur++;
	    	}
  	}
}

// Le traitement consiste à emettre la commande
void traiteCommande()
{
	// Enable wireless reception
	panstamp.rxOn();

	if (passage==0)
	{
		txEmission.commande='6';
		txEmission.longueur=0;
		txEmission.transmet();		
	}
	else
	{
		for (int i=0; i<passage; i++) 
		{
			txEmission.commande=tableaumemoire[i].donnees[0];
			memcpy(txEmission.donnees,&tableaumemoire[i].donnees[1],tableaumemoire[i].longueur-1);
			txEmission.longueur=tableaumemoire[i].longueur-1;
			txEmission.transmet();
			tableaumemoire[i].longueur=0;
			delay(1000);
		}
		passage=0;
	}
	reveil=0;
}
