#include "dompi.h"
#include <Wire.h>
#include <HardwareSerial.h>
#include "timer1a0.h"

/* Constantes */
// le canal RF
#define CANAL           9

// le mot de synchro
#define SYNCWORD1       0xD3
#define SYNCWORD2       0x91

// L'adresse du panstamp côté compteur
#define SOURCE          2

// L'adresse du pantsamp côté box
#define DESTINATION     0

#define I2C_ADDR 0x48
#define TEMP_REG 0x00
#define CONF_REG 0x01

#define TIMER         timer1a0
#define RESET_TIMER()
#define INIT_TIMER()  TIMER.attachInterrupt(isrT1event)
#define START_TIMER() TIMER.start(1000)
#define STOP_TIMER()  TIMER.stop()

/* Variables globales */
// Timer 1 ticks
byte t1Ticks = 0;

// La conf RF
unsigned char conf[] = {CANAL, SYNCWORD1, SYNCWORD2, SOURCE};

// La liste des puissances d'émission dans l'ordre croissant
unsigned char puissances[] = {0x51, 0x8E, 0x60, 0x50, 0x8D, 0x8C, 0x8B, 0x8A, 0x8A, 0xCF, 0x89, 0x88, 0x87, 0x86, 0x85};
bool modesommeill = false;

// le paquet reçu
EMISSION rxEmission = EMISSION(DESTINATION, SOURCE);

// le paquet émis
EMISSION txEmission = EMISSION(SOURCE, DESTINATION);

// indique que le panstamp est en sommeil
int sleep1 = 1;
boolean demande = false;

// La température mesurée
int temperature = 0 ;

// La température de consigne
#define MAX_TEMP	16
char consigne[MAX_TEMP];

// Initialisation des autres constantes
boolean commandeComplete = false;
int consigne1;
int ptemp = 0;
int dtemp = 0;
int compteur = 0;

// Interruption du TIMER
void isrT1event(void)
{
    if (t1Ticks == 10)
    {
        Serial.println("Fin du TIMER : mise en mode sommeil");
	    // Mise en mode sommeil au bout de 10 ticks et RESET
	    sleep1 = 1;
	    RESET_TIMER();
	    STOP_TIMER();
        t1Ticks = 0;
    }
    else
    {
	    t1Ticks++;
    }
}

// Modification du paramétrage RF
void setConfigRF(unsigned char* conf)
{
    // chanel compris entre 0 et 9
    if (conf[0] >= 0 && conf[0] <= 9)
    {
        panstamp.radio.setChannel(conf[0]);
    }
    panstamp.radio.setSyncWord(conf[1], conf[2]);
    // 0 correspond à l'adresse de broadcast. Elle n'est donc pas attribuable
    if (conf[3] > 0)
    {
        panstamp.radio.setDevAddress(conf[3]);
    }
    panstamp.radio.setCCregs();
}
 
// Le traitement pour chaque paquet RF reçu
void rfPacketReceived(CCPACKET *packet)
{
    rxEmission.ajoute(*packet);
}

// Initialisation du panstamp
void setup()
{
    // Initialisation de la conf RF
    setConfigRF(conf);

    // Initialisation du port série
    Serial.begin(57600);
    Wire.begin();

	// La fonction à appeler lors de la réception d'un paquet RF
    panstamp.attachInterrupt(rfPacketReceived);

	// Initialisation du TIMER
	INIT_TIMER();
}

// Se met en mode sommeil
void modeSommeil()
{
    sleep1 = 0;
	// Vide le buffer
    Serial.flush();
    // Mode sommeil court
    if (!modesommeill)
    {
	    panstamp.sleepSec(5);
    }
    // Mode sommeil long 1000 secondes
    else
    {
        panstamp.sleepSec(1000);
    }
}

// Fonction marche chaudière commande 3
void marcheChaudiere()
{
	txEmission.commande = 0x33;
	txEmission.longueur = 0;
	Serial.println("Chaudiere en mode marche");

	traiteCommande();
}

//Fonction arrêt chaudière commande 4
void arretChaudiere()
{
	txEmission.commande = 0x34;
	txEmission.longueur = 0;
	Serial.println("Chaudiere en mode arret");

	traiteCommande();
}

// Fonction qui récupère et envoie la température au panstamp (1)
int envoiTemperature()
{ 
    float degC;
	txEmission.commande = 0x32;
    Wire.beginTransmission(I2C_ADDR);
    Wire.write(TEMP_REG);
    Wire.requestFrom(I2C_ADDR, 2);
    char* tempChar ;
    char tempChar1;
    if (Wire.available() == 2)
    { 
        const byte tempByte1 = Wire.read();
        const byte tempByte2 = Wire.read();
        const int temp16 = (tempByte1 << 4) | (tempByte2 >> 4);
        degC = temp16*0.0625;
        Serial.print("Température actuelle :");
        Serial.println(degC);
    } 
    int degInt = (int)degC;
    txEmission.donnees[0] = degInt;
    float degDec = (degC - degInt) * 100;
    txEmission.donnees[1] = degDec;  
    longueurSerie = sizeof(txEmission.donnees[0])+1;

	traiteCommande();

    return degInt;
}

// Fonction qui régule la température
void reguleTemperature(int consigne, int temperature)
{
	if (temperature < (consigne - 1))
	{
		marcheChaudiere();
	}
	else if (temperature > (consigne + 1))
	{
		arretChaudiere();
	}
    else 
        arretChaudiere();
} 

// Fonction qui indique le réveil au panstamp (1) commande 1
void messageDeReveil()
{
	txEmission.commande = 0x31;
	longueurSerie = 0;

	traiteCommande();         
}

// Fonction de détection d'anomalies 
void detectanomalie(int temperature2)
{
    if (compteur == 0) 
    {
        // 1ère prise de temperature
        ptemp = temperature2;
    }
    else if (compteur == 20)
    {
        // 2ème prise de température
        dtemp = temperature2;

        //Comparaison
        if (dtemp < ptemp - 1)
            anomalie();

        compteur = 0;
    }
}

// Si anomalie détectée, envoi de la commande '8'
void anomalie()
{
    txEmission.commande = 0x38;
    longueurSerie = 0;

    traiteCommande();         
}

// La boucle principale
void loop()
{
    // Traitement du paquet reçu (lorsqu'il est complet)
    if (rxEmission.complet && !rxEmission.traite)
    {
        // Si recoit commande 1 de l'interface --> consigne de l'utilisateur
 		if (rxEmission.commande == '1')
        {
			strcpy(consigne,(char *)rxEmission.donnees);
            consigne1 = atoi(consigne);
			Serial.print("Consigne reçue : ");
			Serial.println(consigne1);
        }
		// Si recoit commande 6 de l'interface --> sleep
        else if (rxEmission.commande == '6')
		{
            sleep1 = 1;
            RESET_TIMER();
            STOP_TIMER();
			Serial.println("Pas de consgine : mode sommeil");
		}
        // Si recoit commande 7 de l'interface --> Départ maison : consigne à 15
        else if (rxEmission.commande == '7')
        {
            consigne1 = 15;
            Serial.println("Consigne a 15");
            Serial.println(consigne1);
        }
        // Si recoit commande 8 de l'interface --> Mode sommeil long
        else if (rxEmission.commande == '8')
        {
            modesommeill = true;
            Serial.println("Mode sommeil long actif");
        }
        // Si recoit commande 9 de l'interface --> Mode sommeil court
        else if (rxEmission.commande == '9')
        {
            modesommeill = false;
            Serial.println("Mode sommeil court actif");
        }
        // Si recoit commande 2 de l'interface --> arret chaudière de l'interface
        else if (rxEmission.commande == '2')
        {
            consigne1 = 0;
            arretChaudiere();
            Serial.println("Arret interface");
        }
        // On est prêt à recevoir un nouveau paquet
        rxEmission.traite = true;
    }
    // Le panstamp est en sommeil 120 secondes au bout de 15 ticks de TIMER 
    if (sleep1) 
    {
	    // Le panstamp entre en sommeil
  	    modeSommeil();
	
		// On compte le temps où il est réveillé en démarrant le TIMER
        START_TIMER();
		
        // Envoie le message de réveil vers panstamp (1)
        messageDeReveil();
		delay(100);

        // Envoi de la température réelle à l'interface
        int temperature = envoiTemperature();

        // Envoi de la régulation de la température au panstamp (3)
        reguleTemperature(consigne1, temperature);

        // Regarde si une anomalie a été détectée
        detectanomalie(temperature);
        compteur++;
    
    }       
    // Si une commande a été lue (et est complète) sur le port série, on l'envoie
    if (commandeComplete)
	{
        traiteCommande();
	}
}

// Le traitement consiste à émettre la commande
void traiteCommande()
{
    txEmission.longueur = longueurSerie;
    txEmission.transmet();
    longueurSerie = 0;
    commandeComplete = false;
}
