#include "structures.h"
#include "panstamp.h"





// L'émission des données avec découpage en plusieurs paquets au besoin
bool EMISSION::transmet(void)
{
	unsigned int i, j;
	bool res;
	// Pas de réception RF pendant que l'on transmet
	panstamp.rxOff();
	

	paquet.data[COMMANDE] = commande;

	// calcul du nombre de paquets à emettre
	paquet.data[NB_PAQUETS] = longueur / (EMETTEUR_MAX_DATA_LEN);
	if (longueur == 0 || longueur % (EMETTEUR_MAX_DATA_LEN) != 0)
	{
		paquet.data[NB_PAQUETS]++;
	}
	// Création des paquets
	for (i = 0; i < paquet.data[NB_PAQUETS]; i++)
	{
		paquet.data[NUM_PAQUET] = i + 1;

		// Calcul de la longueur du paquet courant
		if (i == paquet.data[NB_PAQUETS] - 1)
			// le dernier paquet
			paquet.length = longueur - i*(EMETTEUR_MAX_DATA_LEN) + EMETTEUR_ENTETE_LEN;
		else
			paquet.length = CCPACKET_DATA_LEN;

		// Copie des données dans le paquet
		memcpy(&paquet.data[DONNEES], &donnees[i*(EMETTEUR_MAX_DATA_LEN)], paquet.length - EMETTEUR_ENTETE_LEN + 1);

		// réveil de la RF
		if (i == 0)
		{
			panstamp.radio.wakeUp();
		}
		// emission du paquet
		j = NB_TENTATIVE_EMISSION;
		res = false;
		while (!res && j > 0)
		{
			res = panstamp.radio.sendData(paquet);
			j--;
		}
		if (!res) break;
	}
	// Réactivation de la réception RF
	panstamp.rxOn();
	return res;
}

// Ajout d'un paquet reçu dans les données à traiter
unsigned int EMISSION::ajoute(CCPACKET rxPaquet)
{
	
	// Ce n'est pas la source attendue
	if (adrSrc != rxPaquet.data[ADR_SRC] && adrSrc != 0)
	{
		return 2;
	}
	// Le paquet précédent n'a pas encore été traité
	// FIXME : A priori ce cas n'est pas possible puisque le panstamp bloque les interruptions et donc les réceptions le temps de l'interruption
	//	   et que nous mettons la RF en sommeil le temps de traiter la commande contenu dans le paquet reçu
	//	   A confirmer
	if (!traite)
	{
		return 3;
	}

	// Il s'agit du premier paquet (d'une série peut être)
	// Potentiellement on écrase une série de paquets précédents dont on aurait manqué un paquet
	// Ce cas pourrait être détecté via la variable "complet"
	if (rxPaquet.data[NUM_PAQUET] == 1)
	{
		// Initialisation
		complet = false;
		commande = rxPaquet.data[COMMANDE];
		nbPaquets = rxPaquet.data[NB_PAQUETS];
		longueur = 0;
	}
	// Il s'agit d'un paquet d'une série
	// Mais nous avons perdu au moins un packet précédent de la série
	else if (!complet && (commande != rxPaquet.data[COMMANDE] || numPaquet + 1 != rxPaquet.data[NUM_PAQUET] || nbPaquets != rxPaquet.data[NB_PAQUETS]))
	{
		complet = true;
		return 4;
	}

	numPaquet = rxPaquet.data[NUM_PAQUET];
	// ajout des données reçues dans les données courantes
	memcpy(&donnees[longueur], &rxPaquet.data[DONNEES], rxPaquet.length - EMETTEUR_ENTETE_LEN + 1);
	// Mise à jour de la longueur des données
	longueur += rxPaquet.length - EMETTEUR_ENTETE_LEN;
	// S'il s'agit du dernier paquet
	if (nbPaquets == numPaquet)
	{
		complet = true;
		traite = false;
	}

	return 0;
}

// Le constructeur
EMISSION::EMISSION(unsigned char src, unsigned char dest)
{
	adrSrc = src;
	adrDest = dest;
	paquet.data[ADR_DEST] = adrDest;
	paquet.data[ADR_SRC] = adrSrc;
  longueur = 0;
  complet = false;
  traite = true;
}
